@@ -245,7 +245,7 @@ NCursesColorWindow::NCursesColorWindow(int lines, int cols,
 
 NCursesColorWindow::NCursesColorWindow(NCursesWindow& par,
 		                       int lines, int cols,
-		                       int by, int bx, char absrel = 'a')
+		                       int by, int bx, char absrel)
 : NCursesWindow(par, lines, cols, by, bx, absrel) 
 {
     colorInit();
