@@ -14,18 +14,38 @@
 			  Caltech High Energy Physics
 				 October, 1980
 
+		Hacks to turn this into a test frame for cursor movement:
+			Eric S. Raymond <esr@snark.thyrsus.com>
+				January, 1995
+
+		July 1995 (esr): worms is now in living color! :-)
+
+Options:
+	-f			fill screen with copies of 'WORM' at start.
+	-l <n>			set worm length
+	-n <n>			set number of worms
+	-t			make worms leave droppings
+	-T <start> <end>	set trace interval
+	-S			set single-stepping during trace interval
+	-N			suppress cursor-movement optimization
+
+  This program makes a good torture-test for the ncurses cursor-optimization
+  code.  You can use -T to set the worm move interval over which movement
+  traces will be dumped.  The program stops and waits for one character of
+  input at the beginning and end of the interval.
 */
 
 #include <curses.h>
+#include <stdlib.h>
 #include <signal.h>
 
 #define cursor(col,row) move(row,col)
 
-int Wrap;
 short *ref[128];
-static char flavor[]={
-    'O', '*', '#', '$', '%', '0'
+static chtype flavor[]={
+    'O' , '*', '#', '$', '%', '0', '@',
 };
+#define MAXWORMS	sizeof(flavor)
 static short xinc[]={
      1,  1,  1,  0, -1, -1, -1,  0
 }, yinc[]={
@@ -35,8 +55,14 @@ static struct worm {
     int orientation, head;
     short *xpos, *ypos;
 } worm[40];
+
 static char *field;
-static int length=16, number=3, trail=' ';
+static int length=16, number=3;
+static chtype trail=' ';
+
+#ifdef TRACE
+int generation, trace_start, trace_end, singlestep;
+#endif /* TRACE */
 static struct options {
     int nopts;
     int opts[3];
@@ -123,12 +149,11 @@ static struct options {
     { 0, { 0, 0, 0 } }
 };
 
-void onsig();
-float ranf();
+void onsig(int sig);
+float ranf(void);
 
-main(argc,argv)
-int argc;
-char *argv[];
+int
+main(int argc, char *argv[])
 {
 int x, y;
 int n;
@@ -163,6 +188,18 @@ int last, bottom;
 		case 't':
 		    trail='.';
 		    break;
+#ifdef TRACE
+		case 'S':
+		    singlestep = TRUE;
+		    break;
+		case 'T':
+		    trace_start = atoi(argv[++x]);
+		    trace_end   = atoi(argv[++x]);
+		    break;
+		case 'N':
+		    no_optimize = TRUE;		/* declared by ncurses */
+		    break;
+#endif /* TRACE */
 		default:
 		usage:
 		    fprintf(stderr, "usage: %s [-field] [-length #] [-number #] [-trail]\n",*argv);
@@ -173,25 +210,58 @@ int last, bottom;
 
     signal(SIGINT, onsig);
     initscr();
+#ifdef TRACE
+    noecho();
+    cbreak();
+#endif /* TRACE */
+    nonl();
     bottom = LINES-1;
     last = COLS-1;
 
+#ifdef A_COLOR
+    if (has_colors())
+    {
+	start_color();
+ 
+	init_pair(COLOR_GREEN, COLOR_GREEN, COLOR_BLACK);
+	init_pair(COLOR_RED, COLOR_RED, COLOR_BLACK);
+	init_pair(COLOR_CYAN, COLOR_CYAN, COLOR_BLACK);
+	init_pair(COLOR_WHITE, COLOR_WHITE, COLOR_BLACK);
+	init_pair(COLOR_MAGENTA, COLOR_MAGENTA, COLOR_BLACK);
+	init_pair(COLOR_BLUE, COLOR_BLUE, COLOR_BLACK);
+	init_pair(COLOR_YELLOW, COLOR_YELLOW, COLOR_BLACK);
+
+	flavor[0] |= COLOR_PAIR(COLOR_GREEN) | A_BOLD;
+	flavor[1] |= COLOR_PAIR(COLOR_RED) | A_BOLD;
+	flavor[2] |= COLOR_PAIR(COLOR_CYAN) | A_BOLD;
+	flavor[3] |= COLOR_PAIR(COLOR_WHITE) | A_BOLD;
+	flavor[4] |= COLOR_PAIR(COLOR_MAGENTA) | A_BOLD;
+	flavor[5] |= COLOR_PAIR(COLOR_BLUE) | A_BOLD;
+	flavor[6] |= COLOR_PAIR(COLOR_YELLOW) | A_BOLD;
+    }
+#endif /* A_COLOR */
+
     ip=(short *)malloc(LINES*COLS*sizeof (short));
 
     for (n=0;n<LINES;) {
 		ref[n++]=ip; ip+=COLS;
     }
     for (ip=ref[0],n=LINES*COLS;--n>=0;) *ip++=0;
+
+#ifdef BADCORNER
+    /* if addressing the lower right corner doesn't work in your curses */
     ref[bottom][last]=1;
+#endif /* BADCORNER */
+
     for (n=number, w= &worm[0];--n>=0;w++) {
 		w->orientation=w->head=0;
-		if (!(ip=(short *)malloc(length*sizeof (short)))) {
+		if (!(ip=(short *)malloc((length+1)*sizeof (short)))) {
 		    fprintf(stderr,"%s: out of memory\n",*argv);
 		    exit(1);
 		}
 		w->xpos=ip;
 		for (x=length;--x>=0;) *ip++ = -1;
-		if (!(ip=(short *)malloc(length*sizeof (short)))) {
+		if (!(ip=(short *)malloc((length+1)*sizeof (short)))) {
 		    fprintf(stderr,"%s: out of memory\n",*argv);
 		    exit(1);
 		}
@@ -203,7 +273,7 @@ int last, bottom;
 		p=field;
 		for (y=bottom;--y>=0;) {
 		    for (x=COLS;--x>=0;) {
-				addch(*p++);
+				addch((chtype)(*p++));
 				if (!*p) p=field;
 		    }
             addch('\n');
@@ -212,10 +282,27 @@ int last, bottom;
     refresh();
 
     for (;;) {
+#ifdef TRACE
+		if (trace_start || trace_end) {
+		    if (generation == trace_start) {
+			trace(TRACE_CALLS);
+			getch();
+		    } else if (generation == trace_end) {
+			trace(0);
+			getch();
+		    }
+
+		    if (singlestep && generation > trace_start && generation < trace_end)
+			getch();
+
+		    generation++;
+		}
+#endif /* TRACE */
+
 		for (n=0,w= &worm[0];n<number;n++,w++) {
 		    if ((x=w->xpos[h=w->head])<0) {
 				cursor(x=w->xpos[h]=0,y=w->ypos[h]=bottom);
-				addch(flavor[n%6]);
+				addch(flavor[n % MAXWORMS]);
 				ref[y][x]++;
 		    }
 		    else y=w->ypos[h];
@@ -242,7 +329,9 @@ int last, bottom;
 				w->orientation=op->opts[(int)(ranf()*(float)op->nopts)];
 		    }
 		    cursor(x+=xinc[w->orientation], y+=yinc[w->orientation]);
-		    if (!Wrap||x!=last||y!=bottom) addch(flavor[n%6]);
+
+		    if (y < 0 ) y = 0;
+		    addch(flavor[n % MAXWORMS]);
 		    ref[w->ypos[h]=y][w->xpos[h]=x]++;
 		}
 		refresh();
@@ -250,14 +339,14 @@ int last, bottom;
 }
 
 void
-onsig()
+onsig(int sig)
 {
 	endwin();
-	exit(0);
+	exit(sig);
 }
 
 float
-ranf()
+ranf(void)
 {
 float rv;
 long r = rand();
